@@ -26,8 +26,10 @@ use Password::Policy::Exception::Pwned;
 use Password::Policy::Exception::PwnedError;
 use LWP::UserAgent;
 use Digest::SHA 'sha1_hex';
+use String::Multibyte;
+use Encode 'encode';
 
-our $VERSION = '0.00_03';
+our $VERSION = '0.02_01';
 my $ua = __PACKAGE__ . '/' . $VERSION;
 my $timeout = 5;
 our $base_url = 'https://api.pwnedpasswords.com/range/';
@@ -35,7 +37,9 @@ our $base_url = 'https://api.pwnedpasswords.com/range/';
 sub check {
 	my $self     = shift;
 	my $password = $self->prepare (shift);
-	my $hash     = uc sha1_hex ($password);
+	my $strmb    = String::Multibyte->new ('UTF8');
+	my $hash     = uc sha1_hex ($strmb->islegal($password) ? $password :
+	                    encode ('UTF-8', $password));
 	my $range    = substr ($hash, 0, 5, '');
 	my $url      = $base_url . $range;
 	my $res      = LWP::UserAgent->new (agent => $ua, timeout => $timeout)->get ($url);
@@ -66,16 +70,23 @@ Password::Policy::Rule::Pwned - Check passwords haven't been pwned
     try {
         $pp->process({ password => $pass });
     } catch {
-        warn "This password '$pass' is pwned - don't use it";
+        if ($_->isa ("Password::Policy::Exception::PwnedError")) {
+            warn "Unable to verify pwned status - try again later\n";
+            return;
+        }
+        warn "'$pass' failed checks: $_ - don't use it\n";
         # Other actions
     }
 
 =head1 DESCRIPTION
 
-Plug this rule into L<Password::Policy> to validate potential passwords
-against the list from L<api.pwnedpasswords.com>. It uses the recoomended
-range function to ensure that neither the password nor its full hash is
-ever transferred over the wire.
+Plug this rule into L<Password::Policy> to
+validate potential passwords against the list from
+L<haveibeenpwned.com|https://haveibeenpwned.com/API/v2>. It
+uses the recommended
+L<range|https://haveibeenpwned.com/API/v2#SearchingPwnedPasswordsByRange>
+function to ensure that neither the password nor its full hash is ever
+transferred over the wire.
 
 The Password::Policy configuration file should set the "pwned" attribute
 to 1 in any policy where this rule should apply. A trivial example of
@@ -93,6 +104,22 @@ service to operate it will also throw an exception if the service is
 unavailable for whatever reason. The two exceptions are different and
 may be interrogated to determine the difference.
 
+    try {
+        $pp->process({ password => $pass });
+    } catch {
+        if ($_->isa ("Password::Policy::Exception::Pwned")) {
+            warn "This password '$pass' is pwned - don't use it";
+        } elsif ($_->isa ("Password::Policy::Exception::PwnedError")) {
+            warn "Could not check if password is pwned - use at own risk";
+        } else {
+            warn "Password not pwned but still bad: $_";
+        }
+        # Other actions
+    }
+
+Alternatively the response may be stringified and the messages parsed for
+key phrases, although this will be less robust.
+
     try {
         $pp->process({ password => $pass });
     } catch {
@@ -114,20 +141,15 @@ may be interrogated to determine the difference.
 
 This method is not expected to be called directly but rather via
 C<Password::Policy-E<gt>process>. It takes one argument which is the
-password to be checked. If the password is a utf-8 string it must be
-encoded first.
+password to be checked. The password must be either an encoded utf-8
+bytestring or an unencoded string (which will be encoded internally
+prior to hashing).
 
 The method will throw a L<Password::Policy::Exception::Pwned> exception
 if the password is pwned. If the API server is unavailable it will warn
 and then throw a L<Password::Policy::Exception::PwnedError> exception. It
 will return true if the password is verifiably not pwned.
 
-=head1 STATUS
-
-B<This is beta quality software.> It has been tested and may be used at
-your own risk. The details of the implementation and interface may yet
-be changed in future releases.
-
 =head1 DATA SOURCE
 
 Note that this code is merely a user-friendly API client. It relies
@@ -151,9 +173,9 @@ L<https://gitlab.com/openstrike/password-pwned>
 This module is written and maintained by Pete Houston of Openstrike
 <cpan@openstrike.co.uk>
 
-=head1 COPYRIGHT INFORMATION
+=head1 COPYRIGHT
 
-Copyright 2018 by Pete Houston. All Rights Reserved.
+Copyright 2018, 2024 by Pete Houston. All Rights Reserved.
 
 Permission to use, copy, and  distribute  is  hereby granted,
 providing that the above copyright notice and this permission
@@ -164,5 +186,11 @@ appear in all copies and in supporting documentation.
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
+This means that you can, at your option, redistribute it and/or modify
+it under either the terms of the GNU Public License (GPL) version 1 or
+later, or under the Perl Artistic License.
+
+See L<https://dev.perl.org/licenses/>
+
 =cut
 
